// Create Shortcut
// By Pike

function Favourites(folder)
{
	this.folderFound = false;
	this.init(folder ? folder : 'Favs');
}

Favourites.prototype = {

	MAX_COLLISIONS: 100,
	
	FileFlags: {
		// From http://lxr.mozilla.org/mozilla/source/nsprpub/pr/include/prio.h#600
		RDONLY: 0x01,
		WRONLY: 0x02,
		RDWR: 0x04,
		CREATE_FILE: 0x08,
		APPEND: 0x10,
		TRUNCATE: 0x20,
		SYNC: 0x40,
		EXCL: 0x80
	},
	
	init: function(folder)
	// Get folder to save favourites into
	{
		// First search the directory service for folder
		var dirService = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);
		try
		{
			this.favDir = dirService.get(folder, Components.interfaces.nsILocalFile);
			if (!this.favDir.exists())
			{
				throw 'Folder not found';
			}
		}
		catch (err)
		{
			// Try folder as a pathname
			try
			{
				this.favDir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
				this.favDir.initWithPath(folder);
				if (!this.favDir.exists())
				{
					throw 'Folder not found';
				}
			}
			catch (err)
			{
				this.folderFound = false;
				return false;
			}
		}
		this.folderFound = true;
		return true;
	},
	
	fixName: function(name)
	// Make name a legal favourite name
	{
		name = name.replace(/[\\\/\:\*\?\"\<\>\|]/g, '');
		name = name.replace(/(^[\s]+)|([\s]+$)/g, '');
		return name;
	},
	
	add: function(name, url, overwrite)
	// Add a favourite
	{
		if (this.folderFound)
		{
			name = this.fixName(name);
			var favFile = this.favDir.clone();
			favFile.append(name + '.url');
			if (!overwrite)
			{
				for (var i = 2; favFile.exists() && i < this.MAX_COLLISIONS; i++)
				{
					favFile = this.favDir.clone();
					favFile.append(name + ' (' + i + ').url');
				}
			}
			var aLocalStream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
			aLocalStream.init(favFile, this.FileFlags.WRONLY | this.FileFlags.CREATE_FILE | this.FileFlags.TRUNCATE, 0644, 0);
			var output = new String('[InternetShortcut]\r\nURL=' + url);
			aLocalStream.write(output, output.length);
			aLocalStream.close();
		}
	}
};

function CreateShortcutPrefObserver()
{
	CreateShortcut.setMenuLabel();
}

CreateShortcutPrefObserver.prototype = {

	observe: function(subject, topic, pref)
	{
		if (topic == 'nsPref:changed')
		{
			if (pref == 'prompt')
			{
				CreateShortcut.setMenuLabel();
			}
			else if (pref == 'folder')
			{
				CreateShortcut.initShortcutFolder();
			}
		}
	}
};

var CreateShortcut = {

	DEFAULT_FOLDER: 'DeskP',
	
	onLoad: function()
	{
		// Setup object properties
		this.localeStrings = document.getElementById('createShortcutStrings');
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch('createshortcut.');
		this.initShortcutFolder();
		
		// Setup Event Listener
		var contextMenu = document.getElementById('contentAreaContextMenu');
		contextMenu.addEventListener('popupshowing', function() {CreateShortcut.prepareContextMenu();}, false);
		
		// Setup preference observers
		if (this.prefBranch instanceof Components.interfaces.nsIPrefBranchInternal)
		{
			this.prefObserver = new CreateShortcutPrefObserver();
			this.prefBranch.addObserver('prompt', this.prefObserver, false);
			this.prefBranch.addObserver('folder', this.prefObserver, false);
		}
	},
	
	onUnload: function()
	{
		// Remove preference observer
		if (this.prefBranch instanceof Components.interfaces.nsIPrefBranchInternal)
		{
			this.prefBranch.removeObserver('prompt', this.prefObserver);
			this.prefBranch.removeObserver('folder', this.prefObserver);
		}
	},
	
	initShortcutFolder: function()
	{
		var folder = '';
		try
		{
			folder = this.prefBranch.getComplexValue('folder', Components.interfaces.nsIPrefLocalizedString).data;
		}
		catch (err) {}
		if (!folder)
		{
			folder = this.DEFAULT_FOLDER;
		}
		this.desktopFavs = new Favourites(folder);
	},
	
	prepareContextMenu: function()
	{
		if (gContextMenu)
		{
			// Hide whichever menu item is not needed
			document.getElementById('create-shortcut-from-page').hidden = (gContextMenu.isTextSelected || gContextMenu.onTextInput || gContextMenu.onLink || gContextMenu.onImage);
			document.getElementById('create-shortcut-from-link').hidden = (!gContextMenu.onLink || gContextMenu.onMailtoLink);
		}
	},
	
	setMenuLabel: function()
	// Set value of menuitem label (append ellipsis if prompt dialog is enabled)
	{
		var prompt = true;
		try
		{
			prompt = this.prefBranch.getBoolPref('prompt');
		}
		catch (err) {}
		
		var createLabel = this.localeStrings.getString('createLabel');
		if (prompt)
		{
			createLabel += '...';
		}
		document.getElementById('create-shortcut-from-link').setAttribute('label', createLabel);
		document.getElementById('create-shortcut-from-page').setAttribute('label', createLabel);
	},
	
	createShortcut: function()
	// Get default shortcut values from the current page/link then create shortcut or open dialog
	{
		var name, url;
		
		// Get default values from current page/link
		if (gContextMenu && gContextMenu.onLink)
		{
			name = gContextMenu.linkText();
			url = (typeof gContextMenu.linkURL == 'function' ? gContextMenu.linkURL() : gContextMenu.linkURL);
		}
		else
		{
			name = getBrowser().webNavigation.document.title;
			url = getBrowser().webNavigation.currentURI.spec;
		}
		
		// Get modifiers from prefs
		var prefix = '';
		try
		{
			prefix = this.prefBranch.getComplexValue('prefix', Components.interfaces.nsIPrefLocalizedString).data;
		}
		catch (err) {}
		var suffix = '';
		try
		{
			suffix = this.prefBranch.getComplexValue('suffix', Components.interfaces.nsIPrefLocalizedString).data;
		}
		catch (err) {}
		
		// Calculate shortcut name
		name = this.desktopFavs.fixName(name); // Do manual fixName so 'name' check on next line is accurate
		name = prefix + ' ' + (name ? name : url) + ' ' + suffix;
		
		// Get overwrite pref
		var overwrite = true;
		try
		{
			overwrite = this.prefBranch.getBoolPref('overwrite');
		}
		catch (err) {}
		
		// Create shortcut or open dialog
		var prompt = true;
		try
		{
			prompt = this.prefBranch.getBoolPref('prompt');
		}
		catch (err) {}
		if (prompt)
		{
			window.openDialog('chrome://createshortcut/content/', 'createShortcut', 'chrome,dialog,modal,centerscreen', name, url, this.desktopFavs, overwrite);
		}
		else
		{
			this.desktopFavs.add(name, url, overwrite);
		}
	},
	
	onOpen: function()
	// Set default values for create shortcut dialog
	{
		this.desktopFavs = window.arguments[2];
		this.overwrite = window.arguments[3];
		document.getElementById('name').value = this.desktopFavs.fixName(window.arguments[0]); // Do manual fixName so dialog shows final name
		document.getElementById('url').value = window.arguments[1];
	},
	
	onAccept: function()
	// Final value check then create shortcut
	{
		var name = document.getElementById('name').value;
		var url = document.getElementById('url').value;
		if (!url)
		{
			url = window.arguments[1];
		}
		this.desktopFavs.add(name, url, this.overwrite);
		return true;
	}
};
